# Author -  G05: Cynthia Garcia 404823, Sayani Roy 404791
# Description - The Puzzler is an add-In for Autodesk Fusion 360. It allows you to convert any 
# 3D model into 3D puzzle pieces. Once a 3D model is imported to Fusion 360, you can select the model, 
# specify the complexity of the puzzle and the plugin generates the corresponding puzzle pieces.
 

import adsk.core, adsk.fusion, adsk.cam, traceback
import math
import random
handlers = []

# Global variables
# Define the width of the gaps of the pattern
width_gaps = 0.009

# Inputs for the arc 
pi = math.pi

def run(context):
    ui = None
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Get the CommandDefinitions collection.
        cmdDefs = ui.commandDefinitions
        
        # Create a button command definition.
        buttonSample = cmdDefs.addButtonDefinition('MyButtonDefIdPython', 
                                                   'The Puzzler', 
                                                   'Creates a puzzle.\n\nGenerates the corresponding puzzle from the selected model based on the level of difficulty.',
                                                   './resources/')
        
        buttonSample.toolClipFilename = './resources/ToolClip.png'

        # Connect to the command created event.
        sampleCommandCreated = SampleCommandCreatedEventHandler()
        buttonSample.commandCreated.add(sampleCommandCreated)
        handlers.append(sampleCommandCreated)
        
        # Get the SOLIDS MODIFY panel in the model workspace. 
        solidsPanel = ui.allToolbarPanels.itemById('SolidModifyPanel')
        
        # Add the button to the bottom of split body option.
        buttonControl = solidsPanel.controls.addCommand(buttonSample,'FusionSplitBodyCommand', False)


    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# Event handler for the commandCreated event.
class SampleCommandCreatedEventHandler(adsk.core.CommandCreatedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            app = adsk.core.Application.get()
            ui  = app.userInterface

            eventArgs = adsk.core.CommandCreatedEventArgs.cast(args)
            cmd = eventArgs.command

            # Get manual for the help option
            cmd.helpFile = './manual.pdf'

            # Get the CommandInputs collection to create new command inputs.            
            inputs = cmd.commandInputs
            
            # Select solid input
            selectInput = inputs.addSelectionInput('selection', 'Select Model Body : ', 'Select body for puzzle')
            selectInput.addSelectionFilter('SolidBodies')
            # User can select min 1 element and max 3 elements
            selectInput.setSelectionLimits(1,3)

            # Create dropdown input for difficulty selection 
            # The user can decide to input a number or to use a random number for the creation of the circles
            dropdownInput = inputs.addDropDownCommandInput('level_select', 'Puzzle Level : ', adsk.core.DropDownStyles.LabeledIconDropDownStyle)
            list_items = dropdownInput.listItems
            list_items.add('Easy',True,' ')
            list_items.add('Normal',False,' ')
            list_items.add('Difficult',False,' ')
            
            # Create checkbox input
            checkBoxInput = inputs.addBoolValueInput('c_copy', 'Conserve Original Body : ', True, '', False)

            # Connect to the execute event.
            onExecute = SampleCommandExecuteHandler()
            cmd.execute.add(onExecute)
            handlers.append(onExecute)

        except:
            if ui:
                ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# Event handler for the execute event.
# Creates the cutting pattern for the puzzle based on the selected input.
class SampleCommandExecuteHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            app = adsk.core.Application.get()
            ui  = app.userInterface

            eventArgs = adsk.core.CommandEventArgs.cast(args)

            # Get the values from the command inputs. 
            inputs = eventArgs.command.commandInputs
            
            # Get inputs value of dropdown for the level selection        
            levelInput = inputs.itemById('level_select').selectedItem.name
            
            # Get checkbox if conserve original body from input
            boolCopy = inputs.itemById('c_copy').value

            # Get active document
            design = app.activeProduct
            rootComp = design.rootComponent

            # Create progress dialog and set styles of progress dialog.
            progressDialog = ui.createProgressDialog()
            progressDialog.isBackgroundTranslucent = False
            progressDialog.isCancelButtonShown = False

            # Show process dialog
            progressDialog.show('The Puzzler', ' Generating puzzle (%p%) ', 0, 10, 1)

            # Initialize number of cuts (of rectangle) and arc to make on the pattern
            num_cuts = 0
            num_arcs = 0
            num_arcsXZ = 0
            num_cutsXZ = 0

            # Define number of cuts and arcs based on complexity level
            if levelInput == 'Easy':
                num_cuts = 3
                num_arcs = 1
            elif levelInput == 'Normal':
                num_cuts = 1
                num_arcs = 1
                num_arcsXZ = 2
                num_cutsXZ = 2
            else: 
                num_cuts = 2
                num_arcsXZ = 2
                num_arcs = 1
                
            # Get elements from the input "selection"
            solidInputSelect = inputs.itemById('selection')

            # Save selected elements in list
            solidList = []
            for i in range(solidInputSelect.selectionCount):
                solid = solidInputSelect.selection(i).entity
                # If the solid is large enough, it is saved
                if solid.area > 2.0:
                    solidList.append(solid)

            # Update process dialog value
            progressDialog.progressValue = 1

            # Run cutting tool for each model selected by user
            for solidInput in solidList:

                # Defines the size of the arc based on the area of the input model
                areaT = solidInput.area
                if areaT > 200:
                    size_arc = areaT * 0.00044
                else:
                    size_arc = areaT * 0.0022

                # If Conserve Copy checkbox is enabled, create copy of input body and make it visible
                copy = createCopy(solidInput)
                if boolCopy == False:
                    copy.isVisible = False

                # Cutting pattern for Easy level
                if levelInput == 'Easy':

                    # Update process dialog
                    progressDialog.progressValue += 1

                    # Create first cut in XZ plane
                    bodies1 = createCuttingTool('XZ', solidInput, size_arc, num_arcs, num_cuts)

                    # Check if the new bodies generated are large enough (if they are too small they are hidden)
                    for cutBodies in bodies1.bodies:                  
                        areaT = cutBodies.area
                        if cutBodies.area < 2.0 :
                            cutBodies.isVisible = False

             
                # Cutting pattern for Normal level
                elif levelInput == 'Normal':
                    
                    # Create first cut in XZ plane
                    bodies1 = createCuttingTool('XZ', solidInput, size_arc, num_arcsXZ, num_cutsXZ)
                    
                    # Update process dialog
                    progressDialog.progressValue += 1

                    # Save new bodies created from cut in an array
                    newBodies1 = []

                    # Check if new bodies are large enough for next cut, if they are too small they are skipped
                    for cutBodies in bodies1.bodies:    
                        areaT = cutBodies.area
                        if cutBodies.area > 2.0 :
                            newBodies1.append(cutBodies)
                        else:
                            cutBodies.isVisible = False

                    # Create second cut for the new bodies generated from previous cut 
                    for m in newBodies1:
                        # Update process dialog
                        progressDialog.progressValue += 1

                        # Perform cut in YZ plane
                        bodies2 = createCuttingTool('XY', m, size_arc, num_arcs, num_cuts)

                # Cutting pattern for Difficult level 
                else: 
                   
                    # Create first cut in XZ plane
                    bodies1 = createCuttingTool('XZ', solidInput, size_arc, num_arcsXZ, num_cuts)
                    
                    # Update process dialog
                    progressDialog.progressValue += 1

                    # Save new bodies created from cut in an array
                    newBodies1 = []

                    # Check if new bodies are large enough for next cut, if they are too small they are skipped
                    for cutBodies in bodies1.bodies:
                        areaT = cutBodies.area
                        if cutBodies.area > 2.0 :
                            newBodies1.append(cutBodies)
                        else:
                            cutBodies.isVisible = False

                    # Create second cut for the new bodies generated from previous cut 
                    for m in newBodies1:
                        # Update process dialog
                        progressDialog.progressValue += 1

                        # Perform cut in YZ plane
                        bodies2 = createCuttingTool('YZ', m, size_arc, num_arcs, num_cuts)            

            # Hide process dialog   
            progressDialog.hide()

        except:
            if ui:
                ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# This method performs the cut between the cutting pattern and the body input. It will also calculate the
# requiered information to build the cutting pattern.
# Inputs: The plane in which the pattern should be generated (XZ, YZ, XY), the body to cut, 
# the size of the arc (based on the area of the body) and the number of planes(cuts) and arcs the pattern will have.
# Outputs: The list of bodies generated from the cut.
def createCuttingTool(plane, body, size_arc, num_arcs, num_cuts):
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Get active document.
        design = app.activeProduct

        # Call root
        rootComp = design.rootComponent

        # Get fusion feature tools
        features = rootComp.features

        # Create Tool Object Collection, this variable will save the bodies the final cut with the pattern generated in each plane
        toolBodiesCut = adsk.core.ObjectCollection.create()  

        # Get bounding box position of the input solid
        minPointInput = body.boundingBox.minPoint
        maxPointInput = body.boundingBox.maxPoint

        # Will call the corresponding method to cut the body on the XZ plane
        if plane == "XZ" :
            # Define width of the rectangle in each axis (for the cut)
            distance = adsk.core.ValueInput.createByReal(maxPointInput.y-minPointInput.y)
            
            # Get equally arranged position of the rectangle (for the cut pattern)
            divider_z = (maxPointInput.z - minPointInput.z) / (num_cuts + 1)
            initial_z = minPointInput.z

            # Creates the cutting pattern based on the number of cuts needed
            for i in range(num_cuts):
                # Update position of cutting body
                initial_z += divider_z

                # Construct cutting pattern in XZ plane
                resultBody1 = constructXZPlaneHO(minPointInput, maxPointInput, size_arc, num_arcs, initial_z, distance)
                
                # Adds the resulting body that will be cut in the Object Collection variable
                toolBodiesCut.add(resultBody1)  

        # Will call the corresponding method to cut the body on the YZ plane
        if plane == "YZ" :
            # Define width of the rectangle in each axis (for the cut)
            distance = adsk.core.ValueInput.createByReal(maxPointInput.x-minPointInput.x)

            # Get equally arranged position of the rectangle (for the cut pattern)
            divider_y = (maxPointInput.y - minPointInput.y) / (num_cuts + 1)
            initial_y = minPointInput.y

            # Creates the cutting pattern based on the number of cuts needed
            for i in range(num_cuts):
                # Update position of cutting body
                initial_y += divider_y

                # Construct cutting pattern in YZ plane
                resultBody2 = constructYZPlane(minPointInput, maxPointInput, size_arc, num_arcs, initial_y, distance)
                
                # Adds the resulting body that will be cut in the Object Collection variable
                toolBodiesCut.add(resultBody2)

        # Will call the corresponding method to cut the body on the XY plane 
        if plane == "XY" :
            # Define width of the rectangle in each axis (for the cut)
            distance = adsk.core.ValueInput.createByReal(maxPointInput.z-minPointInput.z)

            # Get equally arranged position of the rectangle (for the cut pattern)
            divider_x = (maxPointInput.x - minPointInput.x) / (num_cuts + 1)
            initial_x = minPointInput.x

            # Creates the cutting pattern based on the number of cuts needed
            for i in range(num_cuts):
                # Update position of cutting body
                initial_x += divider_x

                # Construct cutting pattern in XY
                resultBody3 = constructXYPlane(minPointInput, maxPointInput, size_arc, num_arcs, initial_x, distance)
                
                # Adds the resulting body that will be cut in the Object Collection variable
                toolBodiesCut.add(resultBody3)

        # Get combine feature
        CombineCutFeatsFinal = features.combineFeatures

        # Adds solids to combine feature to perform cut between the body and the cutting pattern
        CombineCutInput_full = rootComp.features.combineFeatures.createInput(body, toolBodiesCut)              

        # Executes cut operation
        CombineCutInput_full.operation = adsk.fusion.FeatureOperations.CutFeatureOperation
        cutBodies = CombineCutFeatsFinal.add(CombineCutInput_full)

        # Return the list of bodies generated from the cut
        return cutBodies

    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# This method draws the sketch of the cutting pattern (in XZ plane-Horizontal) and extrudes the resulting body.
# Input: Min and max points of bounding box of body, arc size (based on the size of the model), number of arcs,
# the initial point to start drawing the sketches and the width for the extrusion of the sketch)
# Output: Returns the body of the cutting pattern.
def constructXZPlaneHO(minPointInput, maxPointInput, size_arc, num_arcs, initial_z, distance):
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Create a document.
        design = app.activeProduct

        # Call root
        rootComp = design.rootComponent

        # Get fusion tools
        features = rootComp.features
        sketches = rootComp.sketches
        extrudes = rootComp.features.extrudeFeatures

        # Create sketch in XZ axis
        sketch = sketches.add(rootComp.xZConstructionPlane)
        
        # Get sketch lines
        sketchLines = sketch.sketchCurves.sketchLines

        #----------Construct the arc body-------------------------------------------------------------------------
        # Set the X coords for arcs 
        div_x_Pt = (maxPointInput.x - minPointInput.x) / (num_arcs + 1)
        init_x_Pt = minPointInput.x
        
        # Generate arcs for sketch
        for j in range(num_arcs):
            if j>0 :
                # Keep a copy of the previous values:
                arc1StartCopy = adsk.core.Point3D.create(arcRadius, -initial_z, minPointInput.y) #end point
            else:
                arc1StartCopy = adsk.core.Point3D.create(minPointInput.x, -initial_z, minPointInput.y) #end point
            
            # Create sketch for arcs
            arcs = sketch.sketchCurves.sketchArcs

            # Updates position of arcs
            init_x_Pt += div_x_Pt

            # Update x coord to get a point inside the rectangle obtained above and generate an closed arc to get profile
            z_increment = initial_z + width_gaps

            # Set points to generate the arc       
            arcRadius = init_x_Pt + size_arc

            # Create points for bigger arc
            arc1Start = adsk.core.Point3D.create(arcRadius, -initial_z, minPointInput.y) 
            arc1End = adsk.core.Point3D.create(init_x_Pt, -initial_z, minPointInput.y) 
            
            # Create points for smaller arc
            arc2Start = adsk.core.Point3D.create(arcRadius, -z_increment, minPointInput.y)
            arc2End = adsk.core.Point3D.create(init_x_Pt, -z_increment, minPointInput.y)

            # Aux variables to define the space between arcs (for the extrusion)
            var_aux1= size_arc+0.07
            var_aux2= size_arc+0.05

            # Generate rand number to alternate positions
            rand_direction = random.randint(0,100)

            # Alternate direction of arc 
            if rand_direction%2 == 0 : 
                arc1Pt = adsk.core.Point3D.create(init_x_Pt, (-z_increment+var_aux1), minPointInput.y)
                arc2Pt = adsk.core.Point3D.create(init_x_Pt, (-z_increment+var_aux2), minPointInput.y)
                arcs.addByThreePoints(arc2Start, arc1Pt, arc2End)
                arcs.addByThreePoints(arc1Start, arc2Pt, arc1End)
            else:
                arc1Pt = adsk.core.Point3D.create(init_x_Pt, (-z_increment-var_aux1), minPointInput.y)
                arc2Pt = adsk.core.Point3D.create(init_x_Pt, (-z_increment-var_aux2), minPointInput.y)
                arcs.addByThreePoints(arc2Start, arc2Pt, arc2End)
                arcs.addByThreePoints(arc1Start, arc1Pt, arc1End)

            
            # Draw two lines to join the end points of the 2 arcs (to have closed shape for extrusion)
            arcJoin1 = sketchLines.addByTwoPoints(arc1Start, arc2Start)
            arcJoin2 = sketchLines.addByTwoPoints(arc1End, arc2End)

            # Sketch Rectangles in between 2 arcs (to have closed shape for extrusion)
            sketchLines.addTwoPointRectangle(arc1StartCopy, arc2End)

            # Sketch a Rectangle after the last arc (to have closed shape for extrusion)
            if j==(num_arcs-1) :
                endRect1 = adsk.core.Point3D.create(init_x_Pt+div_x_Pt, -initial_z - width_gaps, minPointInput.y)
                sketchLines.addTwoPointRectangle(arc1Start, endRect1)
        
        # Creates new Tool Object collection for join of Arc and Rectangle (AR)
        toolBodiesJoinAR = adsk.core.ObjectCollection.create()

        # Extrude the Arcs and the Rectanlges of the sketch
        count_aux = 0
        for profile in sketch.profiles:
            extInputSketch = extrudes.createInput(profile, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)
            extInputSketch.setDistanceExtent(False, distance)
            extInputSketch.isSolid = True
            extSketch = extrudes.add(extInputSketch)

            # Arranges the bodies generated from the extrusion for the join
            if count_aux == 0:
                targetArc = extSketch.bodies.item(0)
            else: 
                toolBodiesJoinAR.add(extSketch.bodies.item(0))
            count_aux += 1

        #----------Combine the rectangle body and the arc body----------------------------------------------------------------

        # Get combine feature
        CombineJoinFeats = features.combineFeatures

        # Set Combine inputs (target body and tool body) rectangle(tool body) with the arc(target body)
        CombineJoinInput = rootComp.features.combineFeatures.createInput(targetArc, toolBodiesJoinAR)              

        # Execute join operation
        CombineJoinInput.operation = adsk.fusion.FeatureOperations.JoinFeatureOperation
        join_body = CombineJoinFeats.add(CombineJoinInput)

        # Return the body generated from join (the cutting pattern)
        return join_body.bodies.item(0)

    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# This method draws the sketch of the cutting pattern (in YZ plane) and extrudes the resulting body.
# Input: Min and max points of bounding box of body, arc size (based on the size of the model), number of arcs,
# the initial point to start drawing the sketches and the width for the extrusion of the sketch)
# Output: Returns the body of the cutting pattern.
def constructYZPlane (minPointInput, maxPointInput, size_arc, num_arcs, initial_y, distance):
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Create a document.
        design = app.activeProduct

        # Call root
        rootComp = design.rootComponent

        # Get fusion tools
        features = rootComp.features
        sketches = rootComp.sketches
        extrudes = rootComp.features.extrudeFeatures

        # Create sketch in YZ axis
        sketch = sketches.add(rootComp.yZConstructionPlane)
        
        # Get sketch lines
        sketchLines = sketch.sketchCurves.sketchLines
        
        #----------Construct the arc body-------------------------------------------------------------------------
        # Set the Z coords for acrs 
        div_z_Pt = (maxPointInput.z - minPointInput.z) / (num_arcs + 1)
        init_z_Pt = minPointInput.z
        
        # Generate arcs for sketch
        for j in range(num_arcs):
            if j>0 :
                # Keep a copy of the previous values:
                arc1StartCopy = adsk.core.Point3D.create(-arcRadius, initial_y, minPointInput.x) #end point
            else:
                arc1StartCopy = adsk.core.Point3D.create(-minPointInput.z, initial_y, minPointInput.x) #end point
            
            # Create sketch for arcs
            arcs = sketch.sketchCurves.sketchArcs

            # Updates position of arcs
            init_z_Pt += div_z_Pt

            # Update Y coord to get a point inside the rectangle obtained above and generate an closed arc to get profile
            y_increment = initial_y + width_gaps

            # Set points to generate the arc       
            arcRadius = init_z_Pt + size_arc
            
            # Create points for bigger arc
            arc1Start = adsk.core.Point3D.create(-arcRadius, initial_y, minPointInput.x) 
            arc1End = adsk.core.Point3D.create(-init_z_Pt, initial_y, minPointInput.x) 

            # Create points for smaller arc
            arc2Start = adsk.core.Point3D.create(-arcRadius, y_increment, minPointInput.x)
            arc2End = adsk.core.Point3D.create(-init_z_Pt, y_increment, minPointInput.x)           
            
            # Aux variables to define the space between arcs (for the extrusion)
            var_aux1= size_arc+0.07
            var_aux2= size_arc+0.05

            # Generate rand number to alternate positions
            rand_direction = random.randint(0,100)

            # Alternate direction of arc 
            if rand_direction%2 == 0 :
                arc1Pt = adsk.core.Point3D.create(-init_z_Pt, (y_increment+var_aux1), minPointInput.x) 
                arc2Pt = adsk.core.Point3D.create(-init_z_Pt, (y_increment+var_aux2), minPointInput.x)
                arcs.addByThreePoints(arc2Start, arc2Pt, arc2End)
                arcs.addByThreePoints(arc1Start, arc1Pt, arc1End)              
            else:
                arc1Pt = adsk.core.Point3D.create(-init_z_Pt, (y_increment-var_aux1), minPointInput.x) 
                arc2Pt = adsk.core.Point3D.create(-init_z_Pt, (y_increment-var_aux2), minPointInput.x)
                arcs.addByThreePoints(arc2Start, arc1Pt, arc2End)
                arcs.addByThreePoints(arc1Start, arc2Pt, arc1End)              
            
            
            # Draw two lines to join the end points of the 2 arcs (to have closed shape for extrusion)
            arcJoin1 = sketchLines.addByTwoPoints(arc1Start, arc2Start)
            arcJoin2 = sketchLines.addByTwoPoints(arc1End, arc2End)

            # Sketch Rectangles in between 2 arcs (to have closed shape for extrusion)
            sketchLines.addTwoPointRectangle(arc1StartCopy, arc2End)

            # Sketch a Rectangle after the last arc (to have closed shape for extrusion)
            if j==(num_arcs-1) :
                endRect1 = adsk.core.Point3D.create(-(init_z_Pt+div_z_Pt), initial_y + width_gaps, minPointInput.x)
                sketchLines.addTwoPointRectangle(arc1Start, endRect1)

        # Creates new Tool Object collection for join of Arc and Rectangle (AR)
        toolBodiesJoinAR = adsk.core.ObjectCollection.create()
        
        # Extrude the Arcs and the Rectanlges of the sketch
        count_aux = 0
        for profile in sketch.profiles:
            extInputSketch = extrudes.createInput(profile, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)
            extInputSketch.setDistanceExtent(False, distance)
            extInputSketch.isSolid = True
            extSketch = extrudes.add(extInputSketch)

            # Arranges the bodies generated from the extrusion for the join
            if count_aux == 0:
                targetArc = extSketch.bodies.item(0)
            else: 
                toolBodiesJoinAR.add(extSketch.bodies.item(0))
            count_aux += 1

        #----------Combine the rectangle body and the arc body----------------------------------------------------------------

        # Get combine feature
        CombineJoinFeats = features.combineFeatures

        # Set Combine inputs (target body and tool body) rectangle(tool body) with the arc(target body)
        CombineJoinInput = rootComp.features.combineFeatures.createInput(targetArc, toolBodiesJoinAR)              

        # Execute join operation
        CombineJoinInput.operation = adsk.fusion.FeatureOperations.JoinFeatureOperation
        join_body = CombineJoinFeats.add(CombineJoinInput)

        # Return the body generated from join (the cutting pattern)
        return join_body.bodies.item(0)
        

    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# This method draws the sketch of the cutting pattern (in XY plane) and extrudes the resulting body.
# Input: Min and max points of bounding box of body, arc size (based on the size of the model), number of arcs,
# the initial point to start drawing the sketches and the width for the extrusion of the sketch)
# Output: Returns the body of the cutting pattern.
def constructXYPlane (minPointInput, maxPointInput, size_arc, num_arcs, initial_x, distance):
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Create a document.
        design = app.activeProduct

        # Call root
        rootComp = design.rootComponent

        # Get fusion tools
        features = rootComp.features
        sketches = rootComp.sketches
        extrudes = rootComp.features.extrudeFeatures

        # Create sketch in different XY axis
        sketch = sketches.add(rootComp.xYConstructionPlane)
        
        # Get sketch lines
        sketchLines = sketch.sketchCurves.sketchLines
        
        #----------Construct the arc body-------------------------------------------------------------------------
        # Set the Z coords for acrs 
        div_y_Pt = (maxPointInput.y - minPointInput.y) / (num_arcs + 1)
        init_y_Pt = minPointInput.y
        
        # Generate arcs for sketch
        for j in range(num_arcs):
            
            if j>0 :
                # Keep a copy of the previous values:
                arc1StartCopy = adsk.core.Point3D.create(initial_x, arcRadius, minPointInput.z) #end point
            else:
                arc1StartCopy = adsk.core.Point3D.create(initial_x, minPointInput.y, minPointInput.z) #end point
            
            # Create sketch for arcs
            arcs = sketch.sketchCurves.sketchArcs

            # Updates position of arcs
            init_y_Pt += div_y_Pt

            # Update Y coord to get a point inside the rectangle obtained above and generate an closed arc to get profile
            x_increment = initial_x + width_gaps

            # Set points to generate the arc       
            arcRadius = init_y_Pt + size_arc

            # Create points for bigger arc
            arc1Start = adsk.core.Point3D.create(initial_x, arcRadius, minPointInput.z) 
            arc1End = adsk.core.Point3D.create(initial_x, init_y_Pt, minPointInput.z) 
                    
            # Create points for smaller arc
            arc2Start = adsk.core.Point3D.create(x_increment, +arcRadius, minPointInput.z)
            arc2End = adsk.core.Point3D.create(x_increment, init_y_Pt, minPointInput.z)
            
            # Aux variables to define the space between arcs (for the extrusion)
            var_aux1= size_arc+0.07
            var_aux2= size_arc+0.05

            # Generate rand number to alternate positions
            rand_direction = random.randint(0,100)

            # Alternate direction of arc 
            if rand_direction%2 == 0 :
                arc1Pt = adsk.core.Point3D.create((x_increment+var_aux1), init_y_Pt, minPointInput.z)
                arc2Pt = adsk.core.Point3D.create((x_increment+var_aux2), init_y_Pt, minPointInput.z)
                arcs.addByThreePoints(arc2Start, arc2Pt, arc2End)
                arcs.addByThreePoints(arc1Start, arc1Pt, arc1End)
            else:
                arc1Pt = adsk.core.Point3D.create((x_increment-var_aux1), init_y_Pt, minPointInput.z)
                arc2Pt = adsk.core.Point3D.create((x_increment-var_aux2), init_y_Pt, minPointInput.z)
                arcs.addByThreePoints(arc2Start, arc1Pt, arc2End)
                arcs.addByThreePoints(arc1Start, arc2Pt, arc1End)

            
            # Draw two lines to join the end points of the 2 arcs (to have closed shape for extrusion)
            arcJoin1 = sketchLines.addByTwoPoints(arc1Start, arc2Start)
            arcJoin2 = sketchLines.addByTwoPoints(arc1End, arc2End)

            # Sketch Rectangles in between 2 arcs (to have closed shape for extrusion)
            sketchLines.addTwoPointRectangle(arc1StartCopy, arc2End)

            # Sketch a Rectangle after the last arc (to have closed shape for extrusion)
            if j==(num_arcs-1) :
                endRect1 = adsk.core.Point3D.create(initial_x + width_gaps, (init_y_Pt + div_y_Pt), minPointInput.z)
                sketchLines.addTwoPointRectangle(arc1Start, endRect1)

        # Creates new Tool Object collection for join of Arc and Rectangle (AR)
        toolBodiesJoinAR = adsk.core.ObjectCollection.create()
        
        # Extrude the Arcs and the Rectanlges of the sketch
        count_aux = 0
        for profile in sketch.profiles:
            extInputSketch = extrudes.createInput(profile, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)
            extInputSketch.setDistanceExtent(False, distance)
            extInputSketch.isSolid = True
            extSketch = extrudes.add(extInputSketch)

            # Arranges the bodies generated from the extrusion for the join
            if count_aux == 0:
                targetArc = extSketch.bodies.item(0)
            else: 
                toolBodiesJoinAR.add(extSketch.bodies.item(0))
            count_aux += 1

        #----------Combine the rectangle body and the arc body----------------------------------------------------------------

        # Get combine feature
        CombineJoinFeats = features.combineFeatures

        # Set Combine inputs (target body and tool body) rectangle(tool body) with the arc(target body)
        CombineJoinInput = rootComp.features.combineFeatures.createInput(targetArc, toolBodiesJoinAR)              

        # Execute join operation
        CombineJoinInput.operation = adsk.fusion.FeatureOperations.JoinFeatureOperation
        join_body = CombineJoinFeats.add(CombineJoinInput)

        # Return the body generated from join (the cutting pattern)
        return join_body.bodies.item(0)
        
    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# This method creates copy of input body (to conserve original body)
# Input: Body to copy
# Output: Copy of the body
def createCopy(body):
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        
        # Gets active document
        design = app.activeProduct

        # Call root
        rootComp = design.rootComponent

        # Check to see if the body is in the root component or another one
        target = None
        if body.assemblyContext:
            target = body.assemblyContext
        else:
            target = rootComp

        # Get the size in X to position copy aside
        xSize = body.boundingBox.maxPoint.x - body.boundingBox.minPoint.x            

        # Creates the copy
        newBody = body.copyToComponent(target)

        # Sets name to the copy body
        newBody.name = "Model Copy"
        
        # Increment the position in X to place the copy       
        currentX = xSize * 1.1

        # Call translation feature to move body
        trans = adsk.core.Matrix3D.create()
        trans.translation = adsk.core.Vector3D.create(currentX, 0, 0)

        # Move the body using a move feature
        bodyColl = adsk.core.ObjectCollection.create()
        bodyColl.add(newBody)
        moveInput = rootComp.features.moveFeatures.createInput(bodyColl, trans)
        moveFeat = rootComp.features.moveFeatures.add(moveInput)

        # Return copy of body
        return newBody
  
    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))

# Stops running the plugin
def stop(context):
    ui = None
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface

        # Clean up the UI.
        cmdDef = ui.commandDefinitions.itemById('MyButtonDefIdPython')
        if cmdDef:
            cmdDef.deleteMe()
            
        solidsPanel = ui.allToolbarPanels.itemById('SolidModifyPanel')
        cntrl = solidsPanel.controls.itemById('MyButtonDefIdPython')
        if cntrl:
            cntrl.deleteMe()

    except:
        if ui:
            ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))
